import argparse


def add_argument_pid(parser: argparse.ArgumentParser, *flags, dest: str = 'pid') -> None:
    if len(flags) <= 0:
        flags = ['-p', '--pid']
    parser.add_argument(
        *flags, dest=dest, action='append', required=False, default=[],
        help=f'Set pids.'
    )


def add_argument_journal(parser: argparse.ArgumentParser, *flags, dest: str = 'journal') -> None:
    if len(flags) <= 0:
        flags = ['-j', '--journal']
    parser.add_argument(
        *flags, dest=dest, action='append', required=False, default=[],
        help=f'Set journals.'
    )


def parse_args_pid_journal(parser: argparse.ArgumentParser, pid_dest: str = 'pid', journal_dest: str = 'journal'):
    args = parser.parse_args()
    return args.__getattribute__(pid_dest), args.__getattribute__(journal_dest)



import re
from typing import Optional
import os.path
import xml.etree.ElementTree as ElementTree

import aiohttp
import logging
from aiofile import async_open
from asyncio import Semaphore

logger = logging.getLogger("downloader")

http_sem = Semaphore(8)
file_sem = Semaphore(512)


async def download_person(pid: str) -> Optional[ElementTree.Element]:
    return await download_item("pid/" + pid + ".xml")


async def download_journal_list(pid: str) -> Optional[ElementTree.Element]:
    return await download_item(pid + "/index.xml")


async def download_journal(pid: str) -> Optional[ElementTree.Element]:
    return await download_item(re.sub(r"\.html$", ".xml", pid))


async def download_item(path: str) -> Optional[ElementTree.Element]:
    save_path = os.path.join("save", path)
    if os.path.isfile(save_path):
        async with file_sem:
            try:
                async with async_open(save_path, 'r') as f:
                    logger.debug("use cache: %s" % save_path)
                    html = await f.read()
                    data = ElementTree.fromstring(html)
                    return data
            except:
                logger.debug(" no cache: %s" % save_path)

    url = "https://dblp.org/" + path
    async with http_sem:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as response:
                    logger.info(" download: %s" % path)
                    html = await response.text()
                    data = ElementTree.fromstring(html)
                    os.makedirs(os.path.dirname(save_path), exist_ok=True)
                    async with async_open(save_path, 'w') as f:
                        await f.write(html)
                    return data
        except Exception as e:
            logger.error("invalid response: %s" % e)
    return None



import abc
import asyncio
import logging

import networkx as nx

from dblp_crawler import Graph

logger = logging.getLogger("graph")


class NetworkxGraph(Graph, metaclass=abc.ABCMeta):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.graph: nx.MultiGraph = nx.MultiGraph()

    def summarize_person(self, a, person):  # 构建summary
        self.graph.add_node(a, person=self.persons[a])  # 把作者信息加进图里

    def summarize_publication(self, a, b, publication):  # 构建summary
        self.graph.add_edge(a, b, key=publication.key(), publication=publication)  # 把边加进图里

    def multi_graph_summary(self):
        """输出一个 networkx.MultiGraph，节点对应作者，每条边对应一篇论文，作者间可有多条边"""
        self.graph = nx.MultiGraph()
        self.summarize()
        return self.graph

    def summary_person(self, person, publications):  # 构建summary
        return dict(
            person=person, publications=list(publications.values())
        )

    def summary_cooperation(self, a, b, publications):  # 构建summary
        return dict(publications=list(publications.values()))

    def graph_summary(self):
        """输出一个 networkx.Graph，节点对应作者，每条边对应多篇论文，作者间仅有一条边"""
        g = self.multi_graph_summary()
        gg = nx.Graph()
        authors = {}
        for (a, b, d) in g.edges(data=True):  # 遍历所有文章
            publication = d['publication']
            # 把文章加进边信息里
            data = gg.get_edge_data(a, b)
            if data is None or "publications" not in data:
                data = {"publications": {}}
            data["publications"][publication.key()] = publication
            gg.add_edge(a, b, **data)
            # 把文章加进作者信息里
            if a not in authors:
                authors[a] = {}
            authors[a][publication.key()] = publication
            if b not in authors:
                authors[b] = {}
            authors[b][publication.key()] = publication
        for pid, publications in authors.items():
            gg.add_node(pid, **self.summary_person(g.nodes[pid]['person'], publications))
        for (a, b, d) in list(gg.edges(data=True)):  # 遍历所有文章
            gg.add_edge(a, b, **self.summary_cooperation(
                g.nodes[a]['person'], g.nodes[b]['person'], d["publications"]
            ))
        return gg


if __name__ == "__main__":
    import logging
    import random
    import matplotlib.pyplot as plt

    logging.basicConfig(level=logging.DEBUG)


    class GG(NetworkxGraph):
        def filter_publications_at_crawler(self, publications):
            publications = list(publications)
            if len(publications) > 0:
                publication = publications[random.randint(0, len(publications) - 1)]
                yield publication

        def filter_publications_at_output(self, publications):
            return self.filter_publications_at_crawler(publications)


    def networkx_drop_noob_once(g: nx.Graph, filter_min_publications=3):
        for node, data in list(g.nodes(data=True)):
            if data is None or 'publications' not in data or len(data['publications']) < filter_min_publications:
                # 文章数量太少？
                g.remove_node(node)  # 应该不是老师吧
        return g


    async def main():
        g = GG(['74/1552-1', '256/5272'], [])
        await g.bfs_once()
        print("-" * 100)
        await g.bfs_once()
        print("-" * 100)

        summary = g.graph_summary()
        for node, data in list(summary.nodes(data=True)):
            print(node, data['person'].name(), len(list(data['person'].publications())))

        fig, ax = plt.subplots(figsize=(12, 12))
        nx.draw(g.graph_summary(), ax=ax)
        ax.margins(0.1, 0.05)
        fig.tight_layout()

        fig, ax = plt.subplots(figsize=(12, 12))
        nx.draw(networkx_drop_noob_once(g.graph_summary()), ax=ax)
        ax.margins(0.1, 0.05)
        fig.tight_layout()

        plt.axis("off")
        plt.show()


    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



import abc
import asyncio
import logging
from typing import Optional

from neo4j import Session

from dblp_crawler import Graph, DBLPPerson, Publication

logger = logging.getLogger("graph")


def add_person(tx, person: DBLPPerson):
    tx.run("MERGE (a:Person {pid: $pid, name: $name})", pid=person.pid(), name=person.name())


def add_edge(tx, a: str, b: str, publication: Publication):
    tx.run("MATCH (a:Person) WHERE a.pid = $a "
           "MATCH (b:Person) WHERE b.pid = $b "
           "MERGE (p:Publication {key:$key, title:$title, journal_key:$journal_key, journal:$journal, year:$year}) "
           "MERGE (a)-[:WRITE]->(p)"
           "MERGE (b)-[:WRITE]->(p)",
           a=a, b=b,
           key=publication.key(),
           title=publication.title(),
           journal_key=publication.journal_key(),
           journal=publication.journal(),
           year=publication.year())


class Neo4jGraph(Graph, metaclass=abc.ABCMeta):
    def __init__(self, session: Session, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.session = session

    def summarize_person(self, a: str, person: Optional[DBLPPerson]):  # 构建summary
        if person is not None:
            self.session.execute_write(add_person, person)  # 把作者信息加进图里

    def summarize_publication(self, a: str, b: str, publication: Publication):  # 构建summary
        self.session.execute_write(add_edge, a, b, publication)  # 把边加进图里


if __name__ == "__main__":
    import logging
    import random
    from neo4j import GraphDatabase

    logging.basicConfig(level=logging.DEBUG)


    class GG(Neo4jGraph):
        def filter_publications_at_crawler(self, publications):
            publications = list(publications)
            if len(publications) > 0:
                publication = publications[random.randint(0, len(publications) - 1)]
                yield publication

        def filter_publications_at_output(self, publications):
            return self.filter_publications_at_crawler(publications)


    async def main():
        with GraphDatabase.driver("neo4j://10.128.202.18:7687") as driver:
            with driver.session() as session:
                g = GG(session, ['74/1552-1', '256/5272'], [])
                await g.bfs_once()
                print("-" * 100)
                await g.bfs_once()
                print("-" * 100)
                g.summarize()


    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



import asyncio
from .downloader import *
from typing import Optional, Iterator
import re
import logging

logger = logging.getLogger("parser")


class Person:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "person", "Should be xml of a person in dblpperson!"
        self.data = data

    def name(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "author":
                return child.text
        return None

    def affiliations(self) -> Iterator[str]:
        for child in self.data:
            if child.tag == "note" and "type" in child.attrib and child.attrib["type"] == "affiliation":
                if child.text is not None:
                    yield child.text

    def __str__(self) -> str:
        name = self.name()
        affiliations = "\n".join(str(a) for a in self.affiliations())
        return "%s\n%s" % (name, affiliations)


class Author:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "author", "Should be xml of a author in someone's publication!"
        self.data = data

    def name(self) -> str:
        return self.data.text if self.data.text is not None else ''

    def pid(self) -> str:
        return self.data.attrib['pid']

    async def dblpperson(self):
        data = await download_person(self.pid())
        if data is None:
            return None
        return DBLPPerson(data)

    def __str__(self) -> str:
        return self.name()


class Publication:
    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "r", "Should be xml of a <r> tag in dblpperson!"
        self.data = data[0]

    def key(self) -> str:
        return self.data.attrib["key"]

    def authors(self) -> Iterator[Author]:
        for child in self.data:
            if child.tag == "author":
                yield Author(child)

    def title(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "title":
                return " ".join(t for t in child.itertext())
        return None

    def journal(self) -> Optional[str]:
        tag = {
            'inproceedings': 'booktitle',
            'proceedings': 'booktitle',
            'article': 'journal',
            'incollection': 'booktitle',
            'book': 'series'
        }
        for child in self.data:
            if self.data.tag in tag and child.tag == tag[self.data.tag]:
                return child.text
        return None

    def journal_key(self) -> Optional[str]:
        for child in self.data:
            if child.tag == "url":
                return "/".join(child.text.split("/")[0:3]) if child.text is not None else None
        return None

    def year(self) -> Optional[int]:
        for child in self.data:
            if child.tag == "year":
                return int(child.text) if child.text is not None else None
        return None

    def ee(self) -> Iterator[str]:
        for child in self.data:
            if child.tag == "ee":
                if child.text is not None:
                    yield child.text

    def doi(self) -> Optional[str]:
        for e in self.ee():
            if re.search(r'doi\.org', e) is not None:
                return e
        return None

    def __str__(self) -> str:
        key = self.key()
        doi = self.doi()
        authors = ", ".join(str(author) for author in self.authors())
        title = self.title()
        journal_year = f"{self.journal()}:{self.year()}"
        return "%s %s\n\t%s\n\t%s\n\t%s" % (key, doi, authors, title, journal_year)


class DBLPPerson:
    ID = 0

    def __init__(self, data: ElementTree.Element) -> None:
        assert data.tag == "dblpperson", "Should be xml of a dblpperson!"
        self.data = data
        logger.debug(f"<{self.data.tag} %s>" % " ".join("%s=\"%s\"" % (k, v) for k, v in self.data.attrib.items()))
        DBLPPerson.ID += 1
        self.id = DBLPPerson.ID

    def pid(self) -> str:
        return self.data.attrib['pid'] if 'pid' in self.data.attrib else ("Unknown_%d" % self.id)

    def name(self) -> str:
        return self.data.attrib['name'] if 'name' in self.data.attrib else ("Unname_%d" % self.id)

    def person(self) -> Optional[Person]:
        for child in self.data:
            if child.tag == "person":
                return Person(child)
        return None

    def publications(self) -> Iterator[Publication]:
        for child in self.data:
            if child.tag == "r":
                yield Publication(child)

    def __str__(self) -> str:
        person = str(self.person())
        publications = "\n".join(str(p) for p in self.publications())
        return "%s\n%s\n\n" % (person, publications)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)


    async def main() -> None:
        print(DBLPPerson(await download_person('74/1552-1')))


    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())
